using namespace utils;
///////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////CONTAINERS/////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////
//Container for molecules
class container
{
  public:
  //calculate position after reflection
  virtual VexT get_position_after_reflection(const VexT& v_init,const VexT& v_fin) =0;
  virtual bool is_in_container(const VexT& v) =0;
  virtual ~container(){};
  virtual void get(double& a,double& b) =0;
  virtual VexT random_initialize_in()=0;
  virtual VexT random_initialize_on()=0; 
  virtual double get_volume()=0;
  gsl_rng* rand_gen;
  container(gsl_rng* i_rand_gen); 
};
container::container(gsl_rng* i_rand_gen) 
          :rand_gen(i_rand_gen)
  {}
////defined for sphere centered in 0,0,0
////particle is assumed to be a point during ,,reaction'' with the sphere
class sphere : public container
{
  public:
  VexT v1,v2;
  exT a,b,c,t0,t1;
  exT r,sq_r;            //radius and sq radius of the sphere
  sphere(const exT&, gsl_rng* init_rand);
  ~sphere();
  VexT get_position_after_reflection(const VexT&,const VexT&);
  bool is_in_container(const VexT& v);
  void get(double& a,double& b) {a=r;b=sq_r;};
  VexT random_initialize_in();
  VexT random_initialize_on();
  double get_volume();
};
sphere::sphere(const exT& init_volume,gsl_rng* init_rand)
      :r(cbrt((3.0*init_volume)/(4.0*M_PI))),sq_r(r*r),v1(3),v2(3),
       container(init_rand)
  {}
sphere::~sphere()
  {}
double sphere::get_volume()
  {
    return (r*r*r*4.0*M_PI/3.0);
  }  
VexT sphere::get_position_after_reflection(const VexT& v_init,const VexT& v_fin)
  { 
    a = gsl_pow_2(v_fin[0] - v_init[0]) + 
        gsl_pow_2(v_fin[1] - v_init[1]) + 
        gsl_pow_2(v_fin[2] - v_init[2]) ;
    b = 2.0*( (v_fin[0]-v_init[0])*v_init[0] +
              (v_fin[1]-v_init[1])*v_init[1] +
              (v_fin[2]-v_init[2])*v_init[2] 
            );
    c = gsl_pow_2(v_init[0])+gsl_pow_2(v_init[1])+gsl_pow_2(v_init[2])
        -gsl_pow_2(r);
    t0= (-b+sqrt(b*b-4.0*a*c))/(2.0*a);
    t1= (-b-sqrt(b*b-4.0*a*c))/(2.0*a);
    v1= v_init+(v_fin-v_init)*t0;
    v2= v_init+(v_fin-v_init)*t1;
    if(dot<3>((v1-v_init),(v_fin-v1))>0.0) return v1;
    else return v2;   
  }
bool sphere::is_in_container(const VexT& v)
{
  if(vec_sq_norm<3>(v)<=sq_r) {return true;}
  else {return false;}
}
VexT sphere::random_initialize_in()
{
  VexT helper(3);
  do {
  for(size_t i(0);i<3;++i){helper[i]=gsl_rng_uniform_pos(rand_gen)*2.0-1.0;}
  }while(vec_sq_norm<3>(helper)>=1.0); 
  return (helper*r);
}
VexT sphere::random_initialize_on()
{
  exT x1,x2;
  VexT helper(3);
  do {
    x1=gsl_rng_uniform_pos(rand_gen)*2.0-1.0;x2=gsl_rng_uniform_pos(rand_gen)*2.0-1.0;
  }while(x1*x1+x2*x2>=1.0); 
  helper[0]=2.0*x1*sqrt(1.0-x1*x1-x2*x2);
  helper[1]=2.0*x2*sqrt(1.0-x1*x1-x2*x2);
  helper[2]=1.0-2.0*(x1*x1+x2*x2);
  return (helper*r);
}
///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////

