/*
 * =====================================================================================
 *        Class:  maple
 *  Description:  input for maple 
 * =====================================================================================
 */
class maple
{

  public:
  maple(const root& r,string i_sim_dir); 
  void write_input_for_maple();
  private:
  void write_reactions();
  void write_opening_and_simulation();
  const root&           r;
  ofstream        ofstr;
  string          sim_dir;


};
maple::maple(const root& i_r,string i_sim_dir)
     :r(i_r), 
      ofstr((i_sim_dir+"input.map").c_str(),ios::trunc),
      sim_dir(i_sim_dir)
{
    ofstr.setf(ios::showpoint);
}
void maple::write_input_for_maple()
{
write_reactions();
write_opening_and_simulation();
}
void maple::write_reactions()
{
  //reactions
  for(size_t i(0);i<r.vect_reagents.size();++i)
  {
    const reagent* rg=&r.vect_reagents[i];
    ofstr<<"eq"<<i<<" :=diff("<<rg->name<<"(t),t)=";
    for(size_t j(0);j<rg->n_substrates;++j)
    {
      ofstr<<"-k"<<rg->as_substrate[j];
      for(size_t k(0);k<r.vect_reactions[rg->as_substrate[j]].n_subs;++k)
      {
        ofstr<<"*"<<r.vect_reagents[r.vect_reactions[rg->as_substrate[j]].subs_ind[k]].name<<"(t)";
      }
    }
    for(size_t j(0);j<rg->n_products;++j)
    {
      ofstr<<"+k"<<rg->as_product[j];
      for(size_t k(0);k<r.vect_reactions[rg->as_product[j]].n_subs;++k)
      {
        ofstr<<"*"<<r.vect_reagents[r.vect_reactions[rg->as_product[j]].subs_ind[k]].name<<"(t)";
      }
    }
    ofstr<<";\n";
  }
  ofstr<<"\n";
  //rate constants
  for(size_t i(0);i<r.vect_reactions.size();++i)
  {
    ofstr<<"k"<<i<<" :="<<r.vect_reactions[i].react_rate_const
         <<";\n";
  }
  ofstr<<"\n";
  //equations
  ofstr<<"eqs :=";
  for(size_t i(0);;)
  {
    ofstr<<"eq"<<i; 
    ++i;
    if(i<r.vect_reagents.size())
    {
      ofstr<<",";
    }
    else {ofstr<<";\n\n";break;}
  }
  //initial concentrations
  ofstr<<"init :=";
  for(size_t i(0);;)
  {
    ofstr<<r.vect_reagents[i].name<<"(0)="<<r.vect_init_molecules[i]<<".0"; 
    ++i;
    if(i<r.vect_reagents.size())
    {
      ofstr<<",";
    }
    else {ofstr<<";\n\n";break;}
  }
  }
void maple::write_opening_and_simulation()
{
  //open_file
  for(size_t i(0);i<r.vect_write_molecules.size();++i)
  {
    ofstr<<"file"<<i<<" := fopen(\""
                    <<sim_dir
                    <<r.vect_reagents[r.vect_write_molecules[i]].name
                    <<"_determ.dat"
                    <<"\", WRITE);\n";
  }
  ofstr<<"\ndsol := dsolve({eqs,init}, numeric, method=rkf45, maxfun=0, output=listprocedure);\n\n";
  for(size_t i(0);i<r.vect_write_molecules.size();++i)
  {
    ofstr<<"f"<<i<<" :=eval("<<r.vect_reagents[r.vect_write_molecules[i]].name<<"(t),dsol);\n";
  }
  //only for one time step from the writing particles
  
  mfm_molecules_write_conc* mfm=dynamic_cast<mfm_molecules_write_conc*>(r.vect_molecules[r.vect_write_molecules[0]]);
  ofstr<<"\nfor t from "<<mfm->s_time<<" by "<<mfm->step_time<<" to "<<mfm->e_time<<" do\n\n";
  //fprint  
  for(size_t i(0);i<r.vect_write_molecules.size();++i)
  {
    ofstr<<"fprintf(file"<<i<<", \"%e %e\\n\",t,f"<<i<<"(t));\n";
  }
  ofstr<<"\nend do;\n\n";
  //close files
  for(size_t i(0);i<r.vect_write_molecules.size();++i)
  {
    ofstr<<"close(file"<<i<<");\n";
  }
}
