/**
 * =====================================================================================\n
 *        Class:  reaction \n
 *  Description:  contains information about reaction \n
 * ===================================================================================== 
 */

class reaction
{
  private:
  static int  numb_reac;      ///<number of reactions
   
  public:
  double            react_rate_const;   ///<react_rate_const
  int               reac_ind;           ///<index of reaction
  bool              if_g;               ///<gillespie method for this reaction with drawing of which molecule
  bool              if_pure_g;          ///<gillespie method
  bool              delayed;            ///<if reaction is delayed
  double            number_of_steps;    ///<number of steps for delayed reaction

  vector<int>       subs_ind;  ///<vector of reagent involved on the left side (ind)
  size_t            n_subs;    ///<number of substrates
  bool              no_simple; ///<if there are two substrates of the same kind (true) 

  vector<int>       prod_ind;  ///<vector of reagent involved on the right side (ind)
  size_t            n_prods;   ///<number of products
  
  reaction (double init_react_rate_const);  /* constructor */
  void add_subs_ind(int i);   
  void add_prod_ind(int i); 
  void ascribe();  
};  /* -----  end of class  reaction  ----- */

/**
 * @brief constructor
 *
 * @param init_react_rate_const  reaction rate constant
 *
 **/
reaction::reaction (double init_react_rate_const)
        : react_rate_const(init_react_rate_const),no_simple(false),if_pure_g(false),if_g(false),delayed(false),number_of_steps(1.0)
{
   reac_ind=numb_reac;
  ++numb_reac;
}
int reaction::numb_reac=0;

/**
 * @brief add index of reaction to the vector of substrate(s) (subs_ind) 
 *
 * @param i  --- index of reaction
 *
 **/
void reaction::add_subs_ind(int i)
  {
    subs_ind.push_back(i);
  }
 /**
 * @brief add index of reaction to the vector of product(s) (prod_ind) 
 *
 * @param i  --- index of reaction
 *
 **/

void reaction::add_prod_ind(int i)  
  {
    prod_ind.push_back(i);
  }

void reaction::ascribe()
  {
    n_subs=subs_ind.size();
    n_prods=prod_ind.size();
  } 

