/**
 * =====================================================================================\n
 *        Class:  reagent \n
 *  Description:  it contains information about molecules \n
 * ===================================================================================== \n
 */
class reagent
{
  private:
  static int        numb_reag;      ///<number of reagents(minus one (pos in vect))
  public:
  string            name;           ///<name of the molecule
  double            diff_const;     ///<diffusion constant
  double            radius;         ///<radius of the reagent
  size_t            group;          ///<group
  
  bool              if_gill;        ///<for reactions with this reagent, gillespie's methods are used 
   
  vector<int>       as_substrate;   ///<vector of reactions involved (ind) [substrate]
  size_t            n_substrates;   ///<size of as_substrate
  vector<int>       as_product;     ///<vector of reactions involved (ind) [product]
  size_t            n_products;     ///<size of as_product
  int               mol_ind;        ///<molecule index

  reagent (string init_name, double init_diff_const, double init_radius);  /* constructor */
  reagent (string init_name);
  void add_as_substrate(int);      
  void add_as_product(int);  
  void ascribe();   
};  /* -----  end of class  reagent  ----- */

/**
 * @brief constructor
 *
 * @param init_name --- name of the molecule 
 * @param init_diff_const  --- diffusion constant 
**/
reagent::reagent (string init_name,double init_diff_const,double init_radius)
        : name(init_name),diff_const(init_diff_const),radius(init_radius)
{
  mol_ind=numb_reag;
  ++numb_reag;
  if_gill=false;
}
reagent::reagent (string init_name)
        : name(init_name),diff_const(0),radius(0)
{
  mol_ind=numb_reag;
  ++numb_reag;
  if_gill=true;
}
int reagent::numb_reag=0;

/**
 * @brief add index of reaction in which reagent takes place as substrate
 *
 * @param i  --- index of reaction
 *
 **/
void reagent::add_as_substrate(int i)
  {
    as_substrate.push_back(i);
  }

/**
 * @brief add index of reaction in which reagent takes place as product
 *
 * @param i  --- index of reaction
 *
 * @return 
 **/
void reagent::add_as_product(int i)
  {
    as_product.push_back(i);
  }
void reagent::ascribe()
  {
    n_substrates=as_substrate.size();
    n_products=as_product.size();
  } 

