namespace utils{
typedef double exT;
typedef valarray<exT> VexT;
template<unsigned DIM>
class sqmat 
   {
     private:
     exT* a_start;
   static exT* builder()
      {
      exT* p = new exT[DIM*DIM];
      if(p) return p; else {cerr<<"sqmat: Out of memory !"<<endl; exit(1);}
      }
   public:
  sqmat()
      :a_start(builder()) {}
  sqmat(const sqmat<DIM>& source)
      :a_start(builder()) {memcpy(a_start,source.a_start,DIM*DIM*sizeof(exT));}
  ~sqmat() {delete[] a_start;}
   const sqmat<DIM>& operator= (const sqmat<DIM>&);
   void clear()
      {for(unsigned i=0; i<DIM; ++i) for(unsigned j=0; j<DIM; ++j) *(a_start+i*DIM+j)=0; }
   const exT& operator()(unsigned col,unsigned row) const
      {assert(row<DIM && col<DIM); return *(a_start+row*DIM+col);}
   exT& operator()(unsigned col,unsigned row)
      {assert(row<DIM && col<DIM); return *(a_start+row*DIM+col);}
   VexT veccol(unsigned col) const 
      {
      VexT result(DIM); 
      for(unsigned row=0; row<DIM; ++row) { result[col]=*(a_start+row*DIM+col) ; }
      return result;
      }
   VexT vecrow(unsigned row) const 
      {
      VexT result(DIM);
      for(unsigned col=0; col<DIM; ++col) { result[col]=*(a_start+row*DIM+col) ; }
      return result;
      }
   void set_veccol(unsigned col, const VexT& source)
      {
      for(unsigned row=0; row<DIM; ++row) { *(a_start+row*DIM+col) = source[row] ; }
      }
   void set_vecrow(unsigned row, const VexT& source)
      {
      for(unsigned col=0; col<DIM; ++col) { *(a_start+row*DIM+col) = source[row] ; }
      }
   sqmat trans() const
      {
      sqmat<DIM> result;
      for(unsigned i=0; i<DIM; ++i) for(unsigned j=0; j<DIM; ++j)
         *(result.a_start+j*DIM+i) = *(a_start+i*DIM+j);
      return result;
      }

      };
template<unsigned DIM>
sqmat<DIM> operator*(const sqmat<DIM>& A, const sqmat<DIM>& B)
   {
   sqmat<DIM> C;
   for(unsigned x=0; x<DIM; ++x) for(unsigned y=0; y<DIM; ++y)
      {
      exT s=0;
      for(unsigned i=0; i<DIM; ++i) s += A(i,y)*B(x,i) ;
      C(x,y)=s;
      }
   return C;
   }
template<unsigned DIM>
exT  dot(const VexT& a, const VexT& b)
   {
   exT ss = 0.0;
   for(unsigned i=0; i<DIM; ++i) ss += a[i]*b[i];
   return ss;
   }
template<unsigned DIM>
sqmat<DIM> Householder_reflection(const VexT& z)
   {
   exT zzf = dot<DIM>(z,z);
   if(double(zzf)==0.0) {cerr<<"Zero norm vector in Householder reflection !"<<endl; exit(1);}
   if( DIM<2 ) {cerr<<"Dimension to low for Householder reflection !"<<endl; exit(1);}
   sqmat<DIM> H;
   zzf = -2.0/zzf;
   for(unsigned x=0; x<DIM; ++x) for(unsigned y=0; y<DIM; ++y)  H(x,y) = zzf * z[x] * z[y] ;
   for(unsigned i=0; i<DIM; ++i) H(i,i) += 1; 
   return H;
   }
template<unsigned DIM>
sqmat<DIM> target_rotator(const VexT& origin, const VexT& target)
   {
   return Householder_reflection<DIM>(target+origin) * Householder_reflection<DIM>(origin);
   }
template<unsigned DIM>
VexT operator* (const sqmat<DIM>& A, const VexT& v)
   {
   VexT result(DIM);
   for(unsigned y=0; y<DIM; ++y)
      {
      exT s=0;
      for(unsigned i=0; i<DIM; ++i) s += A(i,y)*v[i] ;
      result[y]=s;
      }
   return result;
   }
template<unsigned DIM>
ostream& operator<< (ostream& os, const sqmat<DIM>& A)
   {
   //ostream save;
   //save.copyfmt(os);
   os<<scientific<<setprecision(3);
   for(unsigned j=0; j<DIM; ++j)
      {
      os<<"\n|"; for(unsigned i=0; i<DIM; ++i) os<<"            ";
      os<<"   |\n|";
      for(unsigned i=0; i<DIM; ++i)
         {
         os<<setw(12);
         if(A(i,j)==0) os<<"      0     ";
            else if(A(i,j)==1) os<<"      1     ";
               else if(abs(A(i,j))<0.001||abs(A(i,j))>100)
                       os<<scientific<<setprecision(3)<<setw(12)<<A(i,j);
                  else os<<fixed<<setprecision(4)<<setw(12)<<A(i,j);
         }
      os<<"   |\n|"; for(unsigned i=0; i<DIM; ++i) os<<"            ";
      os<<"   |";
      }
   os<<"\n\n"<<endl;
   //os.copyfmt(save);
   return os;
   }
template<unsigned DIM>
ostream& operator<< (ostream& os, const VexT& v)
   {
   //ostream save;
   //save.copyfmt(os);
   os<<"\n[ ";
   for(unsigned j=0; j<DIM; ++j)
      {
      if(j) os<<" , ";
      if(abs(v[j])<0.001||abs(v[j])>100) os<<scientific<<setprecision(7)<<v[j];
         else os<<fixed<<setprecision(8)<<v[j];
      }
   os<<" ]  ";
   //os.copyfmt(save);
   return os;
   }
template<unsigned DIM>
exT vec_sq_norm(const VexT& a)
   {
   exT s=0;
   for(unsigned i=0; i<DIM; ++i) s += a[i]*a[i];
   return s;
   }
} //namespace utils
namespace geo{
using namespace utils;
VexT u(3);         
VexT z_axis(3);
exT dist;          
///////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////CONTAINERS/////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////
//A simulation container for molecules
//class container
//{
//  public:
//  //pure virtual functions
//  //calculate position after reflection
//  virtual VexT get_position_after_reflection(const VexT& v_init,const VexT& v_fin) =0;
//  virtual bool is_in_container(const VexT& v) =0;
//  virtual ~container(){};
//};
//////defined for sphere centered in 0,0,0
//////particle is assumed to be a point during reactions with the sphere
//class sphere : public container
//{
//  public:
//  VexT v1,v2;
//  exT a,b,c,t0,t1;
//  exT r,sq_r;            //radius and sq radius of the sphere
//  sphere(const exT&);
//  ~sphere();
//  VexT get_position_after_reflection(const VexT&,const VexT&);
//  bool is_in_container(const VexT& v);
//};
//sphere::sphere(const exT& init_volume)
//      :r(cbrt((3.0*init_volume)/(4.0*M_PI))),sq_r(r*r),v1(3),v2(3)
//  {}
//sphere::~sphere()
//  {}
//VexT sphere::get_position_after_reflection(const VexT& v_init,const VexT& v_fin)
//  { 
//    a = gsl_pow_2(v_fin[0] - v_init[0]) + 
//        gsl_pow_2(v_fin[1] - v_init[1]) + 
//        gsl_pow_2(v_fin[2] - v_init[2]) ;
//    b = 2.0*( (v_fin[0]-v_init[0])*v_init[0] +
//              (v_fin[1]-v_init[1])*v_init[1] +
//              (v_fin[2]-v_init[2])*v_init[2] 
//            );
//    c = gsl_pow_2(v_init[0])+gsl_pow_2(v_init[1])+gsl_pow_2(v_init[2])
//        -gsl_pow_2(r);
//    t0= (-b+sqrt(b*b-4.0*a*c))/(2.0*a);
//    t1= (-b-sqrt(b*b-4.0*a*c))/(2.0*a);
//    v1= v_init+(v_fin-v_init)*t0;
//    v2= v_init+(v_fin-v_init)*t1;
//    if(dot<3>((v1-v_init),(v_fin-v1))>0.0) return v1;
//    else return v2;   
//  }
//bool sphere::is_in_container(const VexT& v)
//{
//  if(vec_sq_norm<3>(v)<=sq_r) {return true;}
//  else {return false;}
//}
///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////
VexT vec_rotation(const VexT& v,const exT& theta,const exT& phi)
  {
    VexT V(3);
    double x0=sin(theta),x2=sin(phi),
           x1=cos(theta),x3=cos(phi);
    V[0]=v[0]*x3+(v[1]*x1+v[2]*x0)*x2;
    V[1]=-v[0]*x2+(v[1]*x1+v[2]*x0)*x3;
    V[2]=-v[1]*x0+v[2]*x1;
    return V;
  }
void calculate_position(const VexT& r_0,const exT& theta,const exT& phi,const exT& r,VexT& r_fin)
{
  z_axis[0]=0.0;z_axis[1]=0.0;z_axis[2]=1.0; 
  u=r_0;
  dist=sqrt(vec_sq_norm<3>(u));
  u=u/dist;   
  sqmat<3> transf=target_rotator<3>(u,z_axis);
  r_fin=transf.trans()*vec_rotation(z_axis,theta,phi);  
  r_fin=r_fin*r;
} 
////FREE DIFFUSION
//R_O --- initial position
//R --- final position
//D --- sum of diffusion constants
//t --- time of jump
//void free_diffuse(VexT& R_0,VexT& R,D,t)
//{
//  double square_root; //sqrt(2Dt)---variance
//  square_root=sqrt(2.0*D*t);
//  //polozenia czastki po skoku
//  for(unsigned i=0; i<3; ++i) R[i]=R_0[i]+square_root*rand_gauss();  
//}    
//////MUST BE DEFINED
////s_db_ov_da=sqrt(D_B/D_A)
////s_da_ov_db=sqrt(D_A/D_B)
////s_da_mp_db=sqrt(D_A*D_B)
////sum_da_db =(D_A+D_B)
////position of molecule A r_A
////position of molecule B r_B
//void get_R_0(const VexT& r_A,const VexT& r_B,VexT& R_0)
//  {
//    R_0=r_A*s_db_ov_da+r_B*s_da_ov_db;
//  }
//void get_r_0(const VexT& r_A,const VexT& r_B,VexT& r_0)
//  {
//    r_0=r_B-r_A;
//  }
//void position_after_reaction(const VexT& R_0,const VexT& R,const VexT& r_A,const VexT& r_B,VexT& r)
//  {
//    r=1.0/(sum_da_db)*((R-R_0)*s_da_mp_db+r_A*D_B+r_B*D_A);        
//  }
//void calc_R()
//  {
//
//  }
//void calc_r()
//  {
//
//  }
//void get_r(const VexT& R,const VexT& r,VexT& r_A,VexT& r_B)
//  {
//    r_A=(1.0/(sum_da_db))*(R*s_da_mp_db-r*D_A);
//    r_B=r+r_A;
//  }
//
//
} //namespace geo
//int main (void)
//{
//  ofstream output_stream;
//  output_stream.setf(ios::showpoint);
//  output_stream.open("f.dat",ios::trunc);
//  output_stream.precision(18);
//  for(double phi=0.0;phi<2.0*M_PI;phi+=0.1)
//  {
//    for(double theta=0.0;theta<M_PI;theta+=0.1)
//    {
//      geo::VexT r0_A(3);   //
//      geo::VexT r0_B(3);    //
//      r0_A[0]=0.0;r0_A[1]=0.0;r0_A[2]=0.5; //z_axis
//      r0_B[0]=1.0;r0_B[1]=1.0;r0_B[2]=1.0; 
//      geo::calculate_position(r0_A,theta,phi,1.5,r0_B);
//      if(r0_B[2]>0.0)
//      output_stream<<r0_B[0]<<" "<<r0_B[1]<<" "<<r0_B[2]<<" "<<endl;
//    }
//  }
//}
